package cross_variety

import (
	"encoding/json"
	"eta/eta_api/controllers"
	"eta/eta_api/models"
	"eta/eta_api/models/data_manage"
	"eta/eta_api/services/data"
	"eta/eta_api/utils"
	"time"
)

// ClassifyController
// @Description: 跨品种分析分类
type ClassifyController struct {
	controllers.BaseAuthController
}

// List
// @Title 跨品种分析分类列表
// @Description 跨品种分析分类列表接口
// @Param   IsShowMe   query   bool  true       "是否只看我的，true、false"
// @Param   Source   query   int  true       "图表类型，3：跨品种分析，4：滚动跨品种分析"
// @Success 200 {object} data_manage.ChartClassifyListResp
// @router /classify/list [get]
func (c *ClassifyController) List() {
	br := new(models.BaseResponse).Init()
	defer func() {
		c.Data["json"] = br
		c.ServeJSON()
	}()
	resp := new(data_manage.ChartClassifyListResp)

	// 获取当前账号的不可见指标
	noPermissionChartIdMap := make(map[int]bool)
	{
		obj := data_manage.EdbInfoNoPermissionAdmin{}
		confList, err := obj.GetAllChartListByAdminId(c.SysUser.AdminId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "获取失败"
			br.ErrMsg = "获取不可见指标配置数据失败，Err:" + err.Error()
			return
		}
		for _, v := range confList {
			noPermissionChartIdMap[v.ChartInfoId] = true
		}
	}

	isShowMe, _ := c.GetBool("IsShowMe")

	source, _ := c.GetInt("Source")
	if source <= 0 {
		source = utils.CHART_SOURCE_CROSS_HEDGING
	}

	rootList, err := data_manage.GetChartClassifyByParentId(0, utils.CHART_SOURCE_CROSS_HEDGING)
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取数据失败，Err:" + err.Error()
		return
	}

	allChartInfo, err := data_manage.GetChartInfoAll([]int{source})
	if err != nil && err.Error() != utils.ErrNoRow() {
		br.Msg = "获取失败"
		br.ErrMsg = "获取图表信息失败，Err:" + err.Error()
		return
	}

	chartInfoMap := make(map[int][]*data_manage.ChartClassifyItems)
	for _, v := range allChartInfo {
		if !isShowMe {
			chartInfoMap[v.ChartClassifyId] = append(chartInfoMap[v.ChartClassifyId], v)
			continue
		}
		if v.SysUserId != c.SysUser.AdminId {
			continue
		}
		chartInfoMap[v.ChartClassifyId] = append(chartInfoMap[v.ChartClassifyId], v)
	}
	rootChildMap := make(map[int][]*data_manage.ChartClassifyItems)

	// 移除没有图表的分类
	allNodes := make([]*data_manage.ChartClassifyItems, 0)
	for _, v := range rootList {
		if v.SysUserId == c.SysUser.AdminId || c.SysUser.RoleTypeCode == utils.ROLE_TYPE_CODE_ADMIN {
			v.Button.OpButton = true
		}

		rootChildMap[v.ParentId] = append(rootChildMap[v.ParentId], v)
		if existItems, ok := chartInfoMap[v.ChartClassifyId]; ok {
			v.Children = existItems
			allNodes = append(allNodes, v)
		}
	}

	// 移除没有权限的图表
	allNodes = data.HandleNoPermissionChart(allNodes, noPermissionChartIdMap)
	resp.AllNodes = allNodes

	br.Ret = 200
	br.Success = true
	br.Msg = "获取成功"
	br.Data = resp
}

// EditChartClassify
// @Title 修改跨品种分析图表分类
// @Description 修改跨品种分析图表分类接口
// @Param	request	body data_manage.EditChartClassifyReq true "type json string"
// @Success 200 Ret=200 修改成功
// @router /classify/edit [post]
func (c *ClassifyController) EditChartClassify() {
	br := new(models.BaseResponse).Init()
	defer func() {
		c.Data["json"] = br
		c.ServeJSON()
	}()
	var req data_manage.EditChartClassifyReq
	err := json.Unmarshal(c.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}
	if req.ChartClassifyName == "" {
		br.Msg = "请输入分类名称"
		br.IsSendEmail = false
		return
	}

	if req.ChartClassifyId <= 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}

	item, err := data_manage.GetChartClassifyById(req.ChartClassifyId)
	if err != nil {
		br.Msg = "保存失败"
		br.Msg = "获取分类信息失败,Err:" + err.Error()
		return
	}

	source := utils.CHART_SOURCE_CROSS_HEDGING
	if item.ChartClassifyName != req.ChartClassifyName {
		//count, err := data_manage.GetChartClassifyCount(req.ChartClassifyName, item.ParentId, source)
		//if err != nil {
		//	br.Msg = "判断名称是否已存在失败"
		//	br.ErrMsg = "判断名称是否已存在失败，Err:" + err.Error()
		//	return
		//}
		//if count > 0 {
		//	br.Msg = "分类名称已存在，请重新输入"
		//	br.IsSendEmail = false
		//	return
		//}

		err = data_manage.EditChartClassify(req.ChartClassifyId, source, req.ChartClassifyName)
		if err != nil {
			br.Msg = "保存失败"
			br.ErrMsg = "保存失败,Err:" + err.Error()
			return
		}
	}
	br.Ret = 200
	br.Msg = "修改成功"
	br.Success = true
	br.IsAddLog = true
}

// DeleteChartClassifyCheck
// @Title 删除图表检测接口
// @Description 删除图表检测接口
// @Param	request	body data_manage.ChartClassifyDeleteCheckResp true "type json string"
// @Success 200 Ret=200 检测成功
// @router /classify/delete/check [post]
func (c *ClassifyController) DeleteChartClassifyCheck() {
	br := new(models.BaseResponse).Init()
	defer func() {
		c.Data["json"] = br
		c.ServeJSON()
	}()
	var req data_manage.ChartClassifyDeleteCheckReq
	err := json.Unmarshal(c.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.ChartClassifyId < 0 && req.ChartInfoId <= 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}
	var deleteStatus int
	var tipsMsg string
	//删除分类
	if req.ChartClassifyId > 0 && req.ChartInfoId == 0 {
		//判断跨品种分析图表分类下，是否含有图表
		count, err := data_manage.GetChartInfoCountByClassifyId(req.ChartClassifyId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "分类下是否含有图表失败，Err:" + err.Error()
			return
		}

		if count > 0 {
			deleteStatus = 1
			tipsMsg = "该分类下关联图表不可删除"
		}
	}

	if deleteStatus != 1 && req.ChartInfoId == 0 {
		classifyCount, err := data_manage.GetChartClassifyCountByClassifyId(req.ChartClassifyId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "分类下是否含有图表失败，Err:" + err.Error()
			return
		}
		if classifyCount > 0 {
			deleteStatus = 2
			tipsMsg = "确认删除当前目录及包含的子目录吗"
		}
	}
	if deleteStatus == 0 {
		tipsMsg = "可删除，进行删除操作"
	}

	resp := new(data_manage.ChartClassifyDeleteCheckResp)
	resp.DeleteStatus = deleteStatus
	resp.TipsMsg = tipsMsg
	br.Ret = 200
	br.Msg = "检测成功"
	br.Success = true
	br.Data = resp
}

// DeleteChartClassify
// @Title 删除跨品种分析图表分类/图表
// @Description 删除跨品种分析图表分类/图表接口
// @Param	request	body data_manage.DeleteChartClassifyReq true "type json string"
// @Success 200 Ret=200 删除成功
// @router /classify/delete [post]
func (c *ClassifyController) DeleteChartClassify() {
	br := new(models.BaseResponse).Init()
	defer func() {
		c.Data["json"] = br
		c.ServeJSON()
	}()

	sysUser := c.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req data_manage.DeleteChartClassifyReq
	err := json.Unmarshal(c.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.ChartClassifyId < 0 && req.ChartInfoId <= 0 {
		br.Msg = "参数错误"
		br.IsSendEmail = false
		return
	}

	//删除分类
	if req.ChartClassifyId > 0 && req.ChartInfoId == 0 {
		//判断是否含有指标
		count, err := data_manage.GetChartInfoCountByClassifyId(req.ChartClassifyId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "判断名称是否已存在失败，Err:" + err.Error()
			return
		}

		if count > 0 {
			br.Msg = "该目录下存在关联指标，不可删除"
			br.IsSendEmail = false
			return
		}

		err = data_manage.DeleteChartClassify(req.ChartClassifyId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "删除失败,Err:" + err.Error()
			return
		}
	}
	resp := new(data_manage.AddChartInfoResp)
	//删除图表
	if req.ChartInfoId > 0 {
		chartInfo, err := data_manage.GetChartInfoById(req.ChartInfoId)
		if err != nil {
			if err.Error() == utils.ErrNoRow() {
				br.Msg = "图表已删除，请刷新页面"
				br.ErrMsg = "指标不存在,Err:" + err.Error()
				return
			} else {
				br.Msg = "删除失败"
				br.ErrMsg = "删除失败，获取指标信息失败，Err:" + err.Error()
				return
			}
		}
		if chartInfo == nil {
			br.Msg = "图表已删除，请刷新页面"
			return
		}
		//图表操作权限
		ok := data.CheckOpChartPermission(sysUser, chartInfo.SysUserId)
		if !ok {
			br.Msg = "没有该图表的操作权限"
			br.ErrMsg = "没有该图表的操作权限"
			return
		}

		// 获取引用该图表的MyCharts, 用于ES删除
		var myCond string
		var myPars []interface{}
		myCond += ` AND a.chart_info_id = ? `
		myPars = append(myPars, chartInfo.ChartInfoId)
		myCharts, e := data_manage.GetMyChartListGroupByCharyInfoIdAndAdminIdByCondition(myCond, myPars)
		if e != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "获取引用图表的MyChats失败, Err: " + e.Error()
			return
		}
		myIds := make([]int, 0)
		for _, m := range myCharts {
			myIds = append(myIds, m.MyChartId)
		}

		source := chartInfo.Source // 跨品种分析图表(滚动跨品种分析)
		//删除图表及关联指标
		err = data_manage.DeleteChartInfoAndData(chartInfo.ChartInfoId)
		if err != nil {
			br.Msg = "删除失败"
			br.ErrMsg = "删除失败，Err:" + err.Error()
			return
		}
		//删除ES
		{
			go data.EsDeleteChartInfo(chartInfo.ChartInfoId)
			// 删除MY ETA 图表 es数据
			//go data.EsDeleteMyChartInfoByChartInfoId(chartInfo.ChartInfoId)
			go data.EsDeleteMyChartInfoByMyChartIds(myIds)
		}

		var condition string
		var pars []interface{}
		condition += " AND chart_classify_id=? AND source = ? "
		pars = append(pars, chartInfo.ChartClassifyId, source)

		condition += " AND chart_info_id>? ORDER BY create_time ASC LIMIT 1 "
		pars = append(pars, req.ChartInfoId)

		nextItem, err := data_manage.GetChartInfoByCondition(condition, pars)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "删除失败"
			br.ErrMsg = "获取下一级图库信息失败，Err:" + err.Error()
			return
		}

		if nextItem != nil {
			resp.UniqueCode = nextItem.UniqueCode
			resp.ChartInfoId = nextItem.ChartInfoId
		} else {
			var condition string
			var pars []interface{}

			condition += " AND level=1 "
			//pars = append(pars, chartInfo.ChartClassifyId)

			condition += " AND chart_classify_id>? ORDER BY chart_classify_id ASC LIMIT 1 "
			pars = append(pars, chartInfo.ChartClassifyId)

			classifyItem, err := data_manage.GetChartClassifyByCondition(condition, pars)
			if err != nil && err.Error() != utils.ErrNoRow() {
				br.Msg = "删除失败"
				br.ErrMsg = "获取下一级图库分类信息失败，Err:" + err.Error()
				return
			}
			if classifyItem != nil {
				nextItem, err = data_manage.GetNextChartInfo(chartInfo.ChartClassifyId)
				if err != nil && err.Error() != utils.ErrNoRow() {
					br.Msg = "删除失败"
					br.ErrMsg = "获取下一级图库信息失败，Err:" + err.Error()
					return
				}
				if nextItem != nil {
					resp.UniqueCode = nextItem.UniqueCode
					resp.ChartInfoId = nextItem.ChartInfoId
				}
			}
		}
		//新增操作日志
		{
			chartLog := new(data_manage.ChartInfoLog)
			chartLog.ChartName = chartInfo.ChartName
			chartLog.ChartInfoId = req.ChartInfoId
			chartLog.ChartClassifyId = chartInfo.ChartClassifyId
			chartLog.SysUserId = sysUser.AdminId
			chartLog.SysUserRealName = sysUser.RealName
			chartLog.UniqueCode = chartInfo.UniqueCode
			chartLog.CreateTime = time.Now()
			chartLog.Content = string(c.Ctx.Input.RequestBody)
			chartLog.Status = "删除图表"
			chartLog.Method = c.Ctx.Input.URI()
			go data_manage.AddChartInfoLog(chartLog)
		}
	}
	br.Ret = 200
	br.Msg = "删除成功"
	br.Success = true
	br.Data = resp
	br.IsAddLog = true
}

// ChartClassifyMove
// @Title 相关性图表分类移动接口
// @Description 相关性图表分类移动接口
// @Success 200 {object} data_manage.MoveChartClassifyReq
// @router /classify/move [post]
func (c *ClassifyController) ChartClassifyMove() {
	br := new(models.BaseResponse).Init()
	defer func() {
		c.Data["json"] = br
		c.ServeJSON()
	}()

	sysUser := c.SysUser
	if sysUser == nil {
		br.Msg = "请登录"
		br.ErrMsg = "请登录，SysUser Is Empty"
		br.Ret = 408
		return
	}

	var req data_manage.MoveChartClassifyReq
	err := json.Unmarshal(c.Ctx.Input.RequestBody, &req)
	if err != nil {
		br.Msg = "参数解析异常！"
		br.ErrMsg = "参数解析失败，Err:" + err.Error()
		return
	}

	if req.ClassifyId <= 0 {
		br.Msg = "参数错误"
		br.ErrMsg = "分类id小于等于0"
		return
	}
	//判断分类是否存在
	chartClassifyInfo, err := data_manage.GetChartClassifyById(req.ClassifyId)
	if err != nil {
		br.Msg = "移动失败"
		br.ErrMsg = "获取分类信息失败,Err:" + err.Error()
		return
	}
	if chartClassifyInfo.Source != utils.CHART_SOURCE_CROSS_HEDGING {
		br.Msg = "分类异常"
		br.ErrMsg = "分类异常,不是跨品种分析图表的分类"
		return
	}
	updateCol := make([]string, 0)

	//如果有传入 上一个兄弟节点分类id
	if req.PrevClassifyId > 0 {
		//上一个兄弟节点
		prevClassify, err := data_manage.GetChartClassifyById(req.PrevClassifyId)
		if err != nil {
			br.Msg = "移动失败"
			br.ErrMsg = "获取上一个兄弟节点分类信息失败,Err:" + err.Error()
			return
		}
		if prevClassify.Source != utils.CHART_SOURCE_CROSS_HEDGING {
			br.Msg = "上级节点分类异常"
			br.ErrMsg = "上级节点分类异常,不是跨品种分析图表的分类"
			return
		}

		//如果是移动在两个兄弟节点之间
		if req.NextClassifyId > 0 {
			//下一个兄弟节点
			nextClassify, err := data_manage.GetChartClassifyById(req.NextClassifyId)
			if err != nil {
				br.Msg = "移动失败"
				br.ErrMsg = "获取下一个兄弟节点分类信息失败,Err:" + err.Error()
				return
			}
			//如果上一个兄弟与下一个兄弟的排序权重是一致的，那么需要将下一个兄弟（以及下个兄弟的同样排序权重）的排序权重+2，自己变成上一个兄弟的排序权重+1
			if prevClassify.Sort == nextClassify.Sort || prevClassify.Sort == chartClassifyInfo.Sort {
				//变更兄弟节点的排序
				updateSortStr := `sort + 2`
				_ = data_manage.UpdateChartClassifySortByParentId(prevClassify.ParentId, prevClassify.ChartClassifyId, prevClassify.Sort, updateSortStr)
			} else {
				//如果下一个兄弟的排序权重正好是上个兄弟节点的下一层，那么需要再加一层了
				if nextClassify.Sort-prevClassify.Sort == 1 {
					//变更兄弟节点的排序
					updateSortStr := `sort + 1`
					_ = data_manage.UpdateChartClassifySortByParentId(prevClassify.ParentId, 0, prevClassify.Sort, updateSortStr)
				}
			}
		}

		chartClassifyInfo.Sort = prevClassify.Sort + 1
		chartClassifyInfo.ModifyTime = time.Now()
		updateCol = append(updateCol, "Sort", "ModifyTime")

	} else {
		firstClassify, err := data_manage.GetFirstChartClassifyByParentIdAndSource(chartClassifyInfo.ParentId, chartClassifyInfo.Source)
		if err != nil && err.Error() != utils.ErrNoRow() {
			br.Msg = "移动失败"
			br.ErrMsg = "获取跨品种分析图表分类排序第一条的分类信息失败,Err:" + err.Error()
			return
		}
		if firstClassify != nil && firstClassify.Source != utils.CHART_SOURCE_CROSS_HEDGING {
			br.Msg = "上级节点分类异常"
			br.ErrMsg = "上级节点分类异常,不是跨品种分析图表的分类"
			return
		}

		//如果该分类下存在其他分类，且第一个其他分类的排序等于0，那么需要调整排序
		if firstClassify != nil && firstClassify.Sort == 0 {
			updateSortStr := ` sort + 1 `
			_ = data_manage.UpdateChartClassifySortByParentId(firstClassify.ParentId, firstClassify.ChartClassifyId-1, 0, updateSortStr)
		}

		chartClassifyInfo.Sort = 0 //那就是排在第一位
		chartClassifyInfo.ModifyTime = time.Now()
		updateCol = append(updateCol, "Sort", "ModifyTime")
	}

	//更新
	if len(updateCol) > 0 {
		err = chartClassifyInfo.Update(updateCol)
		if err != nil {
			br.Msg = "移动失败"
			br.ErrMsg = "修改失败,Err:" + err.Error()
			return
		}
	}

	br.Ret = 200
	br.Success = true
	br.Msg = "移动成功"
}
